import 'dart:developer';

import 'package:flutter/services.dart';
import 'package:hypersnapsdk_flutter/hv_result.dart';
import 'package:hypersnapsdk_flutter/hyper_snap_sdk_constants.dart';

/// FaceMatchMode enum is used to set the face match mode for face match call
///
/// values - FaceMatchMode.faceId, FaceMatchMode.faceId, FaceMatchMode.faceFace
enum FaceMatchMode { faceId, faceIdFaceString, faceFace }

extension FaceMatchModeExtension on FaceMatchMode {
  String get faceMatchModeString {
    switch (this) {
      case FaceMatchMode.faceFace:
        return 'FACE_FACE';
      case FaceMatchMode.faceIdFaceString:
        return 'FACE_ID_FACE_STRING';
      default:
        return 'FACE_ID';
    }
  }
}

///HVNetworkHelper -  used to make ocr call and face match call
class HVNetworkHelper {
  static const MethodChannel _hvNetworkHelperChannel =
      MethodChannel(HyperSnapSDKConstants.hyperSnapChannel);

  /// makeOCRCall() is used to make an OCR api call using the document image
  ///
  /// params :
  ///     endpoint : type - String, value - endpoint URL which will be called to obtain ocr response
  ///     documentUri : type - String, value - path/uri of document image on which ocr call has to be performed
  ///     parameters : type - Map, value - map/json containing key-value pairs of different fraud/malice checks provided
  ///                                     by Hyperverge.co
  ///     headers : type - Map, value - map/json containing key-value pairs of different headers options provided
  ///     onComplete : type - Function(HVResponse?,HVError?)
  ///
  /// returns : void
  static makeOCRCall(
      {required String endpoint,
      required String documentUri,
      required Map parameters,
      required Map headers,
      required Function(HVResponse?, HVError?) onComplete}) async {
    Map resMap = {};
    HVError? hvError;
    HVResponse? hvResponse;

    try {
      resMap = await HVNetworkHelper.networkHelperMakeOCRCall(
          endpoint, documentUri, parameters, headers);

      Map errorObj = resMap["errorObj"];
      Map resObj = resMap["resultObj"];
      hvError = (errorObj.isEmpty) ? null : HVError.fromMap(errorObj);
      hvResponse = (resObj.isEmpty) ? null : HVResponse.fromMap(resObj);
    } catch (e) {
      log(e.toString());
    }

    onComplete(hvResponse, hvError);
  }

  /// makeFaceMatchCall() is used to make face match api call using face and document image
  ///
  /// params :
  ///     endpoint : type - String, value - endpoint URL which will be called to obtain ocr response
  ///     faceUri : type - String, value - path/uri of face image on which face match call has to be performed
  ///     documentUri : type - String, value - path/uri of document image on which face match call has to be performed
  ///     parameters : type - Map, value - map/json containing key-value pairs of different fraud/malice checks provided
  ///                                     by Hyperverge.co
  ///     headers : type - Map, value - map/json containing key-value pairs of different headers options provided
  ///                                     by Hyperverge.co
  ///     faceMatchMode : type - FaceMatchMode, value - one of [faceId, faceFace, faceIdFaceString] to specify the mode to be used for face match
  ///                     default value - FACE_ID
  ///     onComplete : type - Function(HVResponse?,HVError?)
  ///
  /// returns : void
  static makeFaceMatchCall(
      {required String endpoint,
      required String faceUri,
      required String documentUri,
      required Map parameters,
      required Map headers,
      FaceMatchMode faceMatchMode = FaceMatchMode.faceId,
      required Function(HVResponse?, HVError?) onComplete}) async {
    Map resMap = {};
    HVError? hvError;
    HVResponse? hvResponse;
    try {
      resMap = await HVNetworkHelper.networkHelperMakeFaceMatchCall(
          endpoint, faceUri, documentUri, parameters, headers);

      Map errorObj = resMap["errorObj"];
      Map resObj = resMap["resultObj"];
      hvError = (errorObj.isEmpty) ? null : HVError.fromMap(errorObj);
      hvResponse = (resObj.isEmpty) ? null : HVResponse.fromMap(resObj);
    } catch (e) {
      log(e.toString());
    }

    onComplete(hvResponse, hvError);
  }

  //-------------------------------------deprecated methods------------------------------

  @Deprecated('use makeOCRCall()')
  /// networkHelperMakeOCRCall() is used to make an OCR api call using the document image
  ///
  /// params :
  ///     endpoint : type - String, value - endpoint URL which will be called to obtain ocr response
  ///     documentUri : type - String, value - path/uri of document image on which ocr call has to be performed
  ///     parameters : type - Map, value - map/json containing key-value pairs of different fraud/malice checks provided
  ///                                     by Hyperverge.co
  ///     headers : type - Map, value - map/json containing key-value pairs of different headers options provided
  ///                                     by Hyperverge.co
  /// returns :
  ///     Future<Map>, contains the response obtained after ocr call
  static Future<Map> networkHelperMakeOCRCall(
      String endpoint, String documentUri, Map parameters, Map headers) async {
    final Map map = await _hvNetworkHelperChannel
        .invokeMethod(HyperSnapSDKConstants.networkHelperMakeOCRCall, {
      "endpoint": endpoint,
      "documentUri": documentUri,
      "parameters": parameters,
      "headers": headers
    });
    return map;
  }

  @Deprecated('use makeFaceMatchCall()')
  /// networkHelperMakeFaceMatchCall() is used to make face match api call using face and document image
  ///
  /// params :
  ///     endpoint : type - String, value - endpoint URL which will be called to obtain ocr response
  ///     faceUri : type - String, value - path/uri of face image on which face match call has to be performed
  ///     documentUri : type - String, value - path/uri of document image on which face match call has to be performed
  ///     parameters : type - Map, value - map/json containing key-value pairs of different fraud/malice checks provided
  ///                                     by Hyperverge.co
  ///     headers : type - Map, value - map/json containing key-value pairs of different headers options provided
  ///                                     by Hyperverge.co
  ///     faceMatchMode : type - String, value - one of [FACE_ID, FACE_FACE, FACE_ID_FACE_STRING] to specify the mode to be used for face match
  ///                     default value - FACE_ID
  /// returns :
  ///     Future<Map>, contains the response obtained after face match call
  static Future<Map> networkHelperMakeFaceMatchCall(String endpoint,
      String faceUri, String documentUri, Map parameters, Map headers,
      {String faceMatchMode = "FaceMatchMode.FACE_ID"}) async {
    final Map map = await _hvNetworkHelperChannel
        .invokeMethod(HyperSnapSDKConstants.networkHelperMakeFaceMatchCall, {
      "endpoint": endpoint,
      "faceUri": faceUri,
      "documentUri": documentUri,
      "faceMatchMode": faceMatchMode,
      "parameters": parameters,
      "headers": headers
    });
    return map;
  }
}
