import 'dart:convert';
import 'dart:developer';

import 'package:flutter/services.dart';
import 'package:hypersnapsdk_flutter/hyper_snap_sdk_constants.dart';

/// Region enum is used to set the region
///
/// values - Region.india, Region.asiaPacific, Region.unitedStates, Region.africa
enum Region { india, asiaPacific, unitedStates, africa }

extension RegionExtension on Region {
  String get regionName {
    switch (this) {
      case Region.india:
        return 'INDIA';
      case Region.africa:
        return 'AFRICA';
      case Region.unitedStates:
        return 'UNITED_STATES';
      case Region.asiaPacific:
        return 'ASIA_PACIFIC';
      default:
        return 'INDIA';
    }
  }
}

/// HyperSnapSDK - used to initial the sdk,to config the sdk, and to manage user sessions
class HyperSnapSDK {
  static const MethodChannel _hyperSnapSDKChannel =
      MethodChannel(HyperSnapSDKConstants.hyperSnapChannel);

  /// initialize() is used initialize the hypersnapsdk inside the client's application
  ///
  /// params :
  ///     appID : type - String, value - Provided by hyperverge.co
  ///     appKey : type - String, value - Provided by hyperverge.co
  ///     region : type - Region, value - one of [India, APAC, etc]
  /// returns :
  ///     Future<String> `initialised` if initialization is successful

  static Future<String> initialize(
      String appId, String appKey, Region region) async {
    String result = "Not Initialized";
    try {
      result = await _hyperSnapSDKChannel
          .invokeMethod(HyperSnapSDKConstants.initialize, {
        HyperSnapSDKConstants.appId: appId,
        HyperSnapSDKConstants.appKey: appKey,
        HyperSnapSDKConstants.region: region.regionName
      });
    } catch (e) {
      log(e.toString());
    }
    return result;
  }

  /// initializeWithToken() is used to initilalize the hypersnapsdk with accessToken inside the client's application
  ///
  /// params :
  ///     appID : type - String, value - Provided by hyperverge.co
  ///     accessToken : type - String, value - Provided by hyperverge.co
  ///     region : type - Region, value - one of [India, APAC, etc]
  /// returns :
  ///     Future<String> `initialised` if initialization is successful

  static Future<String> initializeWithToken(
      String appId, String accessToken, Region region) async {
    String result = "Not Initialized";

    try {
      result = await _hyperSnapSDKChannel
          .invokeMethod(HyperSnapSDKConstants.initializeWithToken, {
        HyperSnapSDKConstants.appId: appId,
        HyperSnapSDKConstants.accessToken: accessToken,
        HyperSnapSDKConstants.region: region.regionName
      });
    } catch (e) {
      log(e.toString());
    }
    return result;
  }

  /// setHyperSnapSDKConfig is used to config the sdk options
  ///
  /// params :
  ///     hyperSnapSDKOption : type -  HyperSnapSDKOption,
  /// returns :
  ///     void

  static setHyperSnapSDKConfig(
      {required HyperSnapSDKConfig hyperSnapSDKConfig}) async {
    _hyperSnapSDKConfig = hyperSnapSDKConfig;

    try {
      await _hyperSnapSDKChannel.invokeMethod(
          HyperSnapSDKConstants.hyperSnapSDKConfig,
          _hyperSnapSDKConfig.toMap());
    } catch (e) {
      log(e.toString());
    }
  }

  /// startUserSession() is used to start the user session in the application
  ///
  /// params :
  ///     userID : type - String, value - userID which can be generated by the client
  /// returns :
  ///    Future<bool>, returns true if user session has been started successfully
  static Future<bool> startUserSession(String userID) async {
    bool flag = false;
    try {
      flag = await _hyperSnapSDKChannel.invokeMethod(
          HyperSnapSDKConstants.startUserSession,
          {HyperSnapSDKConstants.userId: userID});
    } catch (e) {
      log(e.toString());
    }
    return flag;
  }

  /// endUserSession() is used by the client to end the session if any session is active
  ///
  /// params : empty
  /// returns : void
  static endUserSession() async {
    try {
      await _hyperSnapSDKChannel
          .invokeMethod(HyperSnapSDKConstants.endUserSession);
    } catch (e) {
      log(e.toString());
    }
  }

  /// sortJSONKeysAlphabetically() is used to sort the keys of the response object alphabetically so that signature checks
  /// and other related checks can be performed in a better/faster way
  ///
  /// params :
  ///     map : type - Map, value - response object
  /// returns :
  ///     Future<String>
  static Future<String> sortJSONKeysAlphabetically(Map map) async {
    String string = "";
    try {
      string = await _hyperSnapSDKChannel.invokeMethod(
          HyperSnapSDKConstants.sortJSONKeysAlphabetically, {"map": map});
    } catch (e) {
      log(e.toString());
    }
    return string;
  }

  //---------------------------------------deprecated setters -------------------------------------
  static HyperSnapSDKConfig _hyperSnapSDKConfig = HyperSnapSDKConfig();

  @Deprecated('Use HyperSnapSDKConfig')

  /// setShouldEnableSSLPinning() is set to true if ssl pinning has to enabled in the application
  ///
  /// params :
  ///     shouldEnableSSLPinning : type - bool, value - true/false
  /// returns :
  ///     void
  static setShouldEnableSSLPinning(bool shouldEnableSSLPinning) async {
    _hyperSnapSDKConfig.shouldEnableSSLPinning = shouldEnableSSLPinning;
    await setHyperSnapSDKConfig(hyperSnapSDKConfig: _hyperSnapSDKConfig);
  }

  @Deprecated('Use HyperSnapSDKConfig')

  /// setShouldUseSignature() is set to true if signature of the images has to be used while involving api calls
  /// in the application
  ///
  /// params :
  ///     shouldUseSignature : type - bool
  /// returns :
  ///     void
  static setShouldUseSignature(bool shouldUseSignature) async {
    _hyperSnapSDKConfig.shouldUseSignature = shouldUseSignature;
    await setHyperSnapSDKConfig(hyperSnapSDKConfig: _hyperSnapSDKConfig);
  }

  @Deprecated('Use HyperSnapSDKConfig')

  /// setHttpTimeoutValues() is used to set connectionTimeout, readTimeout, writeTimeout for the api calls
  ///
  /// params :
  ///     connectTimeout : type - int
  ///     readTimeout : type - int
  ///     writeTimeout : type - int
  /// returns :
  ///     void
  static setHttpTimeoutValues(
      int connectTimeout, int readTimeout, int writeTimeout) async {
    _hyperSnapSDKConfig.timeoutConfig = HVTimeoutConfig(
        connectTimeout: connectTimeout,
        writeTimeout: writeTimeout,
        readTimeout: readTimeout);
    await setHyperSnapSDKConfig(hyperSnapSDKConfig: _hyperSnapSDKConfig);
  }

  @Deprecated('Use HyperSnapSDKConfig')

  /// setShouldReturnRawResponse() is set to true if raw response has to be returned from the plugin
  ///
  /// params :
  ///     shouldReturnRawResponse : type - bool
  /// returns :
  ///     void
  static setShouldReturnRawResponse(bool shouldReturnRawResponse) async {
    _hyperSnapSDKConfig.shouldReturnRawResponse = shouldReturnRawResponse;
    await setHyperSnapSDKConfig(hyperSnapSDKConfig: _hyperSnapSDKConfig);
  }

  @Deprecated('Use HyperSnapSDKConfig')

  /// setHVUIConfig() is used to set the ui config for all the HypersnapSDK Screens
  ///
  /// params :
  ///     hvuiConfig : type - HVUIConfig
  /// returns :
  ///     void
  static setHVUIConfig(HVUIConfig hvuiConfig) async {
    _hyperSnapSDKConfig.hvUIConfig = hvuiConfig;
    await setHyperSnapSDKConfig(hyperSnapSDKConfig: _hyperSnapSDKConfig);
  }
}

/// HyperSnapSDKConfig - used to configure the options for the sdk
class HyperSnapSDKConfig {
  bool? shouldEnableSSLPinning;
  bool? shouldUseSignature;
  bool? shouldReturnRawResponse;
  HVTimeoutConfig? timeoutConfig;
  HVUIConfig? hvUIConfig;

  /// HyperSnapSDKConfig() constructor used to configure the options for the sdk
  ///
  /// params :
  ///     shouldEnableSSLPinning(optional) : set to true if ssl pinning has to enabled in the application, type - bool
  ///     shouldUseSignature(optional) : set to true if signature of the images has to be used while involving api calls, type - bool
  ///     shouldReturnRawResponse(optional) : set to true if raw response has to be returned from the plugin, type - bool
  ///     timeoutConfig(optional) : used to set the timeout values

  HyperSnapSDKConfig(
      {this.shouldEnableSSLPinning,
      this.shouldReturnRawResponse,
      this.shouldUseSignature,
      this.timeoutConfig,
      this.hvUIConfig});

  /// Convert HyperSnapConfig object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperSnapSDKConstants.shouldEnableSSLPinning: shouldEnableSSLPinning,
      HyperSnapSDKConstants.shouldUseSignature: shouldUseSignature,
      HyperSnapSDKConstants.shouldReturnRawResponse: shouldReturnRawResponse,
      HyperSnapSDKConstants.connectTimeout: timeoutConfig?.connectTimeout,
      HyperSnapSDKConstants.readTimeout: timeoutConfig?.readTimeout,
      HyperSnapSDKConstants.writeTimeout: timeoutConfig?.writeTimeout,
      HyperSnapSDKConstants.uiConfigMap: hvUIConfig?.toJSONString()
    };
  }
}

/// HVTimeoutConfig - used to configure the timeout values
class HVTimeoutConfig {
  int connectTimeout;
  int readTimeout;
  int writeTimeout;

  /// HVTimeoutConfig() constructor used to configure the timeout values
  ///
  /// params :
  ///     connectTimeout : used to set connectionTimeout for the api calls, type - int
  ///     readTimeout : used to set readTimeout for the api calls, type - int
  ///     writeTimeout : used to set writeTimeout for the api calls, type - int
  HVTimeoutConfig({
    required this.connectTimeout,
    required this.writeTimeout,
    required this.readTimeout,
  });
}

//HV UI CONFIG CLASSES

class HVUIConfig {
  HVUIColor? hvUIColor;
  HVUIFont? hvUIFont;
  HVUIFontSize? hvUIFontSize;
  HVUIFontWeight? hvUIFontWeight;
  HVUIAnimation? hvUIAnimation;
  HVUIBorderRadius? hvUIBorderRadius;
  HVUIAlignment? hvUIAlignment;
  Map? _uiConfigMap;

  ///HVUIConfig() - used to config all kinds of UI for all hypersnap SDK screens
  HVUIConfig(
      {this.hvUIAlignment,
      this.hvUIAnimation,
      this.hvUIColor,
      this.hvUIBorderRadius,
      this.hvUIFont,
      this.hvUIFontSize,
      this.hvUIFontWeight});

  static HVUIConfig fromMap(Map uiConfigMap) {
    var hvUIConfig = HVUIConfig();
    hvUIConfig._uiConfigMap = uiConfigMap;
    return hvUIConfig;
  }

  /// Convert HVUIConfig object to Map
  String toJSONString() {
    if (_uiConfigMap != null) {
      return json.encode(_uiConfigMap);
    }
    return json.encode({
      HyperSnapSDKConstants.hvUIColor: hvUIColor?.toMap(),
      HyperSnapSDKConstants.hvUIFont: hvUIFont?.toMap(),
      HyperSnapSDKConstants.hvUIFontWeight: hvUIFontWeight?.toMap(),
      HyperSnapSDKConstants.hvUIFontSize: hvUIFontSize?.toMap(),
      HyperSnapSDKConstants.hvUIAnimation: hvUIAnimation?.toMap(),
      HyperSnapSDKConstants.hvUIBorderRadius: hvUIBorderRadius?.toMap(),
      HyperSnapSDKConstants.hvUIAlignment: hvUIAlignment?.toMap(),
    });
  }
}

class HVUIColor {
  String? titleTextColor;
  String? descriptionTextColor;
  String? documentSideHintTextColor;
  String? statusTextColor;
  String? primaryButtonBackgroundColor;
  String? primaryButtonTextColor;
  String? primaryButtonBorderColor;
  String? secondaryButtonBackgroundColor;
  String? secondaryButtonTextColor;
  String? secondaryButtonBorderColor;
  String? captureButtonColor;
  String? retakeMessageColor;
  String? alertTextBoxTextColor;
  String? alertTextBoxBorderColor;
  String? alertTextBoxBackgroundColor;

  HVUIColor(
      {this.alertTextBoxBackgroundColor,
      this.alertTextBoxBorderColor,
      this.alertTextBoxTextColor,
      this.captureButtonColor,
      this.descriptionTextColor,
      this.documentSideHintTextColor,
      this.primaryButtonBackgroundColor,
      this.primaryButtonBorderColor,
      this.primaryButtonTextColor,
      this.retakeMessageColor,
      this.secondaryButtonBackgroundColor,
      this.secondaryButtonBorderColor,
      this.secondaryButtonTextColor,
      this.statusTextColor,
      this.titleTextColor});

  /// Convert HVUIColor object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperSnapSDKConstants.titleTextColor: titleTextColor,
      HyperSnapSDKConstants.descriptionTextColor: descriptionTextColor,
      HyperSnapSDKConstants.documentSideHintTextColor:
          documentSideHintTextColor,
      HyperSnapSDKConstants.statusTextColor: statusTextColor,
      HyperSnapSDKConstants.primaryButtonBackgroundColor:
          primaryButtonBackgroundColor,
      HyperSnapSDKConstants.primaryButtonTextColor: primaryButtonTextColor,
      HyperSnapSDKConstants.primaryButtonBorderColor: primaryButtonBorderColor,
      HyperSnapSDKConstants.secondaryButtonBackgroundColor:
          secondaryButtonBackgroundColor,
      HyperSnapSDKConstants.secondaryButtonTextColor: secondaryButtonTextColor,
      HyperSnapSDKConstants.secondaryButtonBorderColor:
          secondaryButtonBorderColor,
      HyperSnapSDKConstants.captureButtonColor: captureButtonColor,
      HyperSnapSDKConstants.retakeMessageColor: retakeMessageColor,
      HyperSnapSDKConstants.alertTextBoxTextColor: alertTextBoxTextColor,
      HyperSnapSDKConstants.alertTextBoxBorderColor: alertTextBoxBorderColor,
      HyperSnapSDKConstants.alertTextBoxBackgroundColor:
          alertTextBoxBackgroundColor,
    };
  }
}

class HVUIFont {
  String? titleTextFont;
  String? descriptionTextFont;
  String? statusTextFont;
  String? documentSideHintTextFont;
  String? retakeMessageFont;
  String? primaryButtonTextFont;
  String? secondaryButtonTextFont;
  String? alertTextBoxTextFont;

  HVUIFont(
      {this.alertTextBoxTextFont,
      this.descriptionTextFont,
      this.documentSideHintTextFont,
      this.primaryButtonTextFont,
      this.retakeMessageFont,
      this.secondaryButtonTextFont,
      this.statusTextFont,
      this.titleTextFont});

  /// Convert HVUIFont object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperSnapSDKConstants.titleTextFont: titleTextFont,
      HyperSnapSDKConstants.descriptionTextFont: descriptionTextFont,
      HyperSnapSDKConstants.statusTextFont: statusTextFont,
      HyperSnapSDKConstants.documentSideHintTextFont: documentSideHintTextFont,
      HyperSnapSDKConstants.retakeMessageFont: retakeMessageFont,
      HyperSnapSDKConstants.primaryButtonTextFont: primaryButtonTextFont,
      HyperSnapSDKConstants.secondaryButtonTextFont: secondaryButtonTextFont,
      HyperSnapSDKConstants.alertTextBoxTextFont: alertTextBoxTextFont,
    };
  }
}

class HVUIFontWeight {
  String? titleTextWeight;
  String? descriptionTextWeight;
  String? statusTextWeight;
  String? documentSideHintTextWeight;
  String? retakeMessageWeight;
  String? primaryButtonTextWeight;
  String? secondaryButtonTextWeight;
  String? alertTextBoxTextWeight;

  HVUIFontWeight(
      {this.alertTextBoxTextWeight,
      this.descriptionTextWeight,
      this.documentSideHintTextWeight,
      this.primaryButtonTextWeight,
      this.retakeMessageWeight,
      this.secondaryButtonTextWeight,
      this.statusTextWeight,
      this.titleTextWeight});

  /// Convert HVUIFontWeight object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperSnapSDKConstants.titleTextWeight: titleTextWeight,
      HyperSnapSDKConstants.descriptionTextWeight: descriptionTextWeight,
      HyperSnapSDKConstants.statusTextWeight: statusTextWeight,
      HyperSnapSDKConstants.documentSideHintTextWeight:
          documentSideHintTextWeight,
      HyperSnapSDKConstants.retakeMessageWeight: retakeMessageWeight,
      HyperSnapSDKConstants.primaryButtonTextWeight: primaryButtonTextWeight,
      HyperSnapSDKConstants.secondaryButtonTextWeight:
          secondaryButtonTextWeight,
      HyperSnapSDKConstants.alertTextBoxTextWeight: alertTextBoxTextWeight,
    };
  }
}

class HVUIFontSize {
  double? titleTextSize;
  double? descriptionTextSize;
  double? statusTextSize;
  double? documentSideHintTextSize;
  double? retakeMessageTextSize;
  double? primaryButtonTextSize;
  double? secondaryButtonTextSize;
  double? alertTextBoxTextSize;

  HVUIFontSize(
      {this.alertTextBoxTextSize,
      this.descriptionTextSize,
      this.documentSideHintTextSize,
      this.primaryButtonTextSize,
      this.retakeMessageTextSize,
      this.secondaryButtonTextSize,
      this.statusTextSize,
      this.titleTextSize});

  /// Convert HVUIFontSize object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperSnapSDKConstants.titleTextSize: titleTextSize,
      HyperSnapSDKConstants.descriptionTextSize: descriptionTextSize,
      HyperSnapSDKConstants.statusTextSize: statusTextSize,
      HyperSnapSDKConstants.documentSideHintTextSize: documentSideHintTextSize,
      HyperSnapSDKConstants.retakeMessageTextSize: retakeMessageTextSize,
      HyperSnapSDKConstants.primaryButtonTextSize: primaryButtonTextSize,
      HyperSnapSDKConstants.secondaryButtonTextSize: secondaryButtonTextSize,
      HyperSnapSDKConstants.alertTextBoxTextSize: alertTextBoxTextSize,
    };
  }
}

class HVUIAnimation {
  String? documentInstructionLottie;
  String? documentInstructionFrontSideLottie;
  String? documentInstructionBackSideLottie;
  String? documentProcessingLottie;
  String? documentSuccessLottie;
  String? documentFailureLottie;
  String? faceInstructionLottie;
  String? faceProcessingLottie;
  String? faceSuccessLottie;
  String? faceFailureLottie;
  String? endStateProcessing;
  String? endStateSuccess;
  String? endStateFailure;
  String? uploadFailure;
  String? qrInstruction;

  HVUIAnimation(
      {this.documentFailureLottie,
      this.documentInstructionBackSideLottie,
      this.documentInstructionFrontSideLottie,
      this.documentInstructionLottie,
      this.documentProcessingLottie,
      this.documentSuccessLottie,
      this.endStateFailure,
      this.endStateProcessing,
      this.endStateSuccess,
      this.faceFailureLottie,
      this.faceInstructionLottie,
      this.faceProcessingLottie,
      this.faceSuccessLottie,
      this.qrInstruction,
      this.uploadFailure});

  /// Convert HVUIAnimation object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperSnapSDKConstants.documentInstructionLottie:
          documentInstructionLottie,
      HyperSnapSDKConstants.documentInstructionFrontSideLottie:
          documentInstructionFrontSideLottie,
      HyperSnapSDKConstants.documentInstructionBackSideLottie:
          documentInstructionBackSideLottie,
      HyperSnapSDKConstants.documentProcessingLottie: documentProcessingLottie,
      HyperSnapSDKConstants.documentSuccessLottie: documentSuccessLottie,
      HyperSnapSDKConstants.documentFailureLottie: documentFailureLottie,
      HyperSnapSDKConstants.faceInstructionLottie: faceInstructionLottie,
      HyperSnapSDKConstants.faceProcessingLottie: faceProcessingLottie,
      HyperSnapSDKConstants.faceSuccessLottie: faceSuccessLottie,
      HyperSnapSDKConstants.faceFailureLottie: faceFailureLottie,
      HyperSnapSDKConstants.endStateProcessing: endStateProcessing,
      HyperSnapSDKConstants.endStateSuccess: endStateSuccess,
      HyperSnapSDKConstants.endStateFailure: endStateFailure,
      HyperSnapSDKConstants.uploadFailure: uploadFailure,
      HyperSnapSDKConstants.qrInstruction: qrInstruction,
    };
  }
}

class HVUIBorderRadius {
  double? primaryButtonRadius;
  double? secondaryButtonRadius;

  HVUIBorderRadius({this.primaryButtonRadius, this.secondaryButtonRadius});

  /// Convert HVUIButtonRadius object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperSnapSDKConstants.primaryButtonRadius: primaryButtonRadius,
      HyperSnapSDKConstants.secondaryButtonRadius: secondaryButtonRadius,
    };
  }
}

class HVUIAlignment {
  String? titleTextAlignment;
  String? descriptionTextAlignment;
  String? statusTextAlignment;

  HVUIAlignment(
      {this.descriptionTextAlignment,
      this.statusTextAlignment,
      this.titleTextAlignment});

  /// Convert HVUIAlignment object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperSnapSDKConstants.titleTextAlignment: titleTextAlignment,
      HyperSnapSDKConstants.descriptionTextAlignment: descriptionTextAlignment,
      HyperSnapSDKConstants.statusTextAlignment: statusTextAlignment,
    };
  }
}
